AC_DEFUN([AC_CHECK_NCURSES],
[
	AC_SUBST(NCURSES_LIBS)
	AC_SUBST(NCURSES_CPPFLAGS)
	
	AC_ARG_WITH(ncurses-dir, 
		AS_HELP_STRING(--with-ncurses-dir,Use path to ncurses installation), [
		if ! test -z "$with_ncurses_dir"; then
			NCURSES_CPPFLAGS="-I$with_ncurses_dir/include"
			NCURSES_LIBS="-L$with_ncurses_dir/lib"
		fi
	])

	save_CPPFLAGS="$CPPFLAGS"
	save_LDFLAGS="$LDFLAGS"
	CPPFLAGS="$CPPFLAGS $NCURSES_CPPFLAGS"
	LDFLAGS="$LDFLAGS $NCURSES_LIBS"
	have_ncurses_h=""
	AC_CHECK_HEADERS([ncurses.h],
	[
		have_ncurses_h="yes"
	], [
		AC_CHECK_HEADERS([ncurses/ncurses.h],
		[
			have_ncurses_h="yes"
		])
	])

	if test "x$have_ncurses_h" = "xyes"; then
		# Checks for general libraries.
		AC_CHECK_LIB(xcurses, tgetent, [have_xcurses=yes], [have_xcurses=no])
		AC_CHECK_LIB(tinfow, tgetent, [have_tinfow=yes], [have_tinfow=no])
		AC_CHECK_LIB(ncursesw, tgetent, [have_ncursesw=yes], [have_ncursesw=no])
		AC_CHECK_LIB(tinfo, tgetent, [have_tinfo=yes], [have_tinfo=no])
		AC_CHECK_LIB(ncurses, tgetent, [have_ncurses=yes], [have_ncurses=no])
		AC_CHECK_LIB(curses, tgetent, [have_curses=yes], [have_curses=no])

		# Checks for terminal libraries
		AC_MSG_CHECKING([for working terminal libraries])
		TERMLIBS=

		dnl -- Try tinfow.
		if test "x$TERMLIBS" = x; then
		  if test $have_tinfow = yes; then
			TERMLIBS="-ltinfow"
			SAVE_LIBS=$LIBS
			LIBS="$LIBS $TERMLIBS"
			AC_TRY_LINK(, [tgetent(0,0); tgetflag(0); tgetnum(0); tgetstr(0,0);],
			  [termok=yes], [termok=no])
			LIBS=$SAVE_LIBS
			if test $termok = no; then TERMLIBS=""; fi
		  fi
		fi

		dnl -- Try ncursesw.
		if test "x$TERMLIBS" = x; then
		  if test $have_ncursesw = yes; then
			TERMLIBS="-lncursesw"
			SAVE_LIBS=$LIBS
			LIBS="$LIBS $TERMLIBS"
			AC_TRY_LINK(, [tgetent(0,0); tgetflag(0); tgetnum(0); tgetstr(0,0);],
			  [termok=yes], [termok=no])
			LIBS=$SAVE_LIBS
			if test $termok = no; then TERMLIBS=""; fi
		  fi
		fi

		dnl -- Try tinfo.
		if test "x$TERMLIBS" = x; then
		  if test $have_tinfo = yes; then
			TERMLIBS="-ltinfo"
			SAVE_LIBS=$LIBS
			LIBS="$LIBS $TERMLIBS"
			AC_TRY_LINK(, [tgetent(0,0); tgetflag(0); tgetnum(0); tgetstr(0,0);],
			  [termok=yes], [termok=no])
			LIBS=$SAVE_LIBS
			if test $termok = no; then TERMLIBS=""; fi
		  fi
		fi

		# -- Try ncurses.
		if test "x$TERMLIBS" = x; then
		  if test $have_ncurses = yes; then
			TERMLIBS="-lncurses"
			SAVE_LIBS=$LIBS
			LIBS="$LIBS $TERMLIBS"
			AC_TRY_LINK(, [tgetent(0,0); tgetflag(0); tgetnum(0); tgetstr(0,0);],
			  [termok=yes], [termok=no])
			LIBS=$SAVE_LIBS
			if test $termok = no; then TERMLIBS=""; fi
		  fi
		fi

		# -- Try xcurses.
		if test "x$TERMLIBS" = x; then
		  if test $have_xcurses = yes; then
			TERMLIBS="-lxcurses"
			SAVE_LIBS=$LIBS
			LIBS="$LIBS $TERMLIBS"
			AC_TRY_LINK(, [tgetent(0,0); tgetflag(0); tgetnum(0); tgetstr(0,0);],
			  [termok=yes], [termok=no])
			LIBS=$SAVE_LIBS
			if test $termok = no; then TERMLIBS=""; fi
		  fi
		fi

		# -- Try curses.
		if test "x$TERMLIBS" = x; then
		  if test $have_curses = yes; then
			TERMLIBS="-lcurses"
			SAVE_LIBS=$LIBS
			LIBS="$LIBS $TERMLIBS"
			AC_TRY_LINK(, [tgetent(0,0); tgetflag(0); tgetnum(0); tgetstr(0,0);],
			  [termok=yes], [termok=no])
			LIBS=$SAVE_LIBS
			if test $termok = no; then TERMLIBS=""; fi
		  fi
		fi

		# -- Try curses & termcap.
		if test "x$TERMLIBS" = x; then
		  if test $have_curses = yes; then
		  if test $have_termcap = yes; then
			TERMLIBS="-lcurses -ltermcap"
			SAVE_LIBS=$LIBS
			LIBS="$LIBS $TERMLIBS"
			AC_TRY_LINK(, [tgetent(0,0); tgetflag(0); tgetnum(0); tgetstr(0,0);],
			  [termok=yes], [termok=no])
			LIBS=$SAVE_LIBS
			if test $termok = no; then TERMLIBS=""; fi
		  fi
		  fi
		fi


		# summarize
		if test "x$TERMLIBS" = x; then
		  AC_MSG_RESULT(Cannot find terminal libraries - configure failed)
		  exit 1
		fi
		AC_MSG_RESULT(using $TERMLIBS)
		NCURSES_LIBS="$NCURSES_LIBS $TERMLIBS"
		have_ncurses="yes"
	fi

	$as_unset have_ncurses_h termok TERMLIBS

	CPPFLAGS="$save_CPPFLAGS"
	LDFLAGS="$save_LDFLAGS"
])
