/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2006 Jimmy Wennlund <jimmy.wennlund@gmail.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#include <initng.h>


#include <stdio.h>
#include <string.h>							/* strstr() */
#include <stdlib.h>							/* free() exit() */
#include <assert.h>

#include <initng_handler.h>
#include <initng_global.h>
#include <initng_plugin_hook.h>
#include <initng_common.h>
#include <initng_toolbox.h>
#include <initng_static_states.h>
#include <initng_env_variable.h>

s_entry CONFLICT = { "conflict", STRINGS, NULL,
	"If service put here is starting or running, bail out."
};

a_state_h CONFLICTING = { "FAILED_BY_CONFLICT", IS_FAILED, NULL, NULL, NULL };

static int check_conflict(active_db_h * service)
{
	const char *conflict_entry = NULL;
	s_data *itt = NULL;

	assert(service);
	assert(service->name);

	/* Do this check when this service is put in a STARTING state */
	if (!IS_STARTING(service))
		return (TRUE);

	/* make sure the conflict entry is set */
	while ((conflict_entry = get_next_string(&CONFLICT, service, &itt)))
	{
		active_db_h *s = NULL;

		/*D_("Making sure that %s is not running.\n", conflict_entry); */
		char *fixed = fix_variables(conflict_entry, service);

		s = initng_active_db_find_by_name(fixed);

		free(fixed);
		/* this is actually good */
		if (!s)
		{
			/*D_("Conflict not found!\n"); */
			continue;
		}

		if (IS_UP(s) || IS_STARTING(s))
		{
			initng_common_mark_service(service, &CONFLICTING);
			F_("Service \"%s\" is conflicting with service \"%s\"!\n",
			   service->name, s->name);
			return (FALSE);
		}
	}
	return (TRUE);
}

int module_init(int api_version)
{
	D_("module_init();\n");
	if (api_version != API_VERSION)
	{
		F_("This module is compiled for api_version %i version and initng is compiled on %i version, won't load this module!\n", API_VERSION, api_version);
		return (FALSE);
	}

	initng_service_data_types_add(&CONFLICT);
	initng_plugin_hook_add(&g.IS_CHANGE, 10, &check_conflict);
	initng_active_state_add(&CONFLICTING);

	return (TRUE);
}

void module_unload(void)
{

	initng_plugin_hook_del(&g.IS_CHANGE, &check_conflict);
	initng_active_state_del(&CONFLICTING);
	initng_service_data_types_del(&CONFLICT);
}
