/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2006 Elan Ruusame <glen@pld-linux.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#include <initng.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#include <initng_handler.h>
#include <initng_global.h>
#include <initng_plugin_hook.h>
#include <initng_common.h>
#include <initng_depend.h>
#include <initng_toolbox.h>
#include <initng_static_data_id.h>
#include <initng_static_states.h>
#include <initng_static_service_types.h>

s_entry LOCKFILE = { "lockfile", SET, NULL,
	"If this option is set, plugin will create /var/lock/subsys/$NAME lockfile."
};

#define LOCKDIR "/var/lock/subsys/"

static int status_change(active_db_h * service)
{
	D_("status change [%s]\n", service->name);

	// are we under influence of lockfile?
	if (is(&LOCKFILE, service))
	{
		char *p = strrchr(service->name, '/') + 1;
		char lockfile[sizeof(LOCKDIR) + strlen(p)];

		strcpy(mempcpy(lockfile, LOCKDIR, sizeof(LOCKDIR) - 1), p);

		D_("lockfile path [%s]\n", lockfile);
		// service states from initng_is.h
		if (IS_UP(service))
		{
			D_("service got up\n");
			creat(lockfile, S_IREAD);

		}
		else if (IS_DOWN(service))
		{
			D_("service went down\n");
			unlink(lockfile);
		}
	}

	return (TRUE);
}

int module_init(int api_version)
{
	D_("module_init();\n");
	if (api_version != API_VERSION)
	{
		F_("This module is compiled for api_version %i version and initng is compiled on %i version, won't load this module!\n", API_VERSION, api_version);
		return (FALSE);
	}

	initng_service_data_types_add(&LOCKFILE);
	initng_plugin_hook_add(&g.IS_CHANGE, 50, &status_change);
	return (TRUE);
}

void module_unload(void)
{
	D_("module_unload();\n");
	initng_service_data_types_del(&LOCKFILE);
	initng_plugin_hook_del(&g.IS_CHANGE, &status_change);
}
